<?php
  // Cron job for census report
  // Currently does nothing
  
  // Cron function
  function report_census_cron() {
      global $CFG;
      $cronjob = true;
      require($CFG->dirroot.'/admin/report/census/index.php');
  }
?>