<?php
$string['census'] = 'Census';
$string['curlerror'] = 'CURL not found. You must have CURL installed to use remote reporting.';
$string['dbupdated'] = 'Database successfully upgraded';
$string['error_noroles'] = 'You must define both teacher and student roles';
$string['form_phonehome'] = 'Allow anonymous reporting of data';
$string['form_remotesystem'] = 'Remote reporting server';
$string['form_roleassignment'] = 'Role assignment';
$string['reportsettings'] = 'Census report settings';
$string['runcensus'] = 'Run census';
?>
